# process tooltips (must be \tooltip{text}{tip} all on one line; can be more than one tip per line
# tex must be of length one
processText = function(tex, texRange) {
  assure(length(tex)==1, 221956)
  
  # Separate mathJax to avoid, e.g., "\bf" ==> "<strong>" inside math (and change "$foo$" to "\(foo\)")
  temp = gregexpr("([$])?([$])([^$])+([$])([$])?", tex)[[1]]
  hasMathJax = length(temp)>1 || temp != -1
  if (hasMathJax) {
    vec = NULL
    vBegin = as.numeric(temp)
    vEnd = vBegin + attr(temp, "match.length") - 1
    vLen = length(vBegin)
    texLen = nchar(tex)
    if (vBegin[1] > 1) vec = substring(tex, 1, vBegin[1]-1)
    for (i in 1:vLen) {
      tempS = substring(tex, vBegin[i], vEnd[i])
      tempL = nchar(tempS)
      if (substring(tempS,2,2) != "$" && substring(tempS, tempL-1, tempL-1) != "$")
        tempS = paste0("\\(", substring(tempS, 2, tempL-1), "\\)")
      vec = c(vec, tempS)
      if (i < vLen) {
        if (vEnd[i+1] > vBegin[i]+1) {
          vec = c(vec, substring(tex, vEnd[i]+1, vBegin[i+1]-1))
        }
      } else { # at last mathJax string
        if (vEnd[i] < texLen) {
          vec = c(vec, substring(tex, vEnd[i]+1, texLen))
        }
      }
    } # end for each mathJax string
    tex = vec
  } # end if any mathJax

  # hold out math from the next set of changes
  if (hasMathJax) {
    holdPos = grep("^([$$]|[\\][(]).*([$$]|[\\][)])$", tex)
    holdTex = tex[holdPos]
    tex[holdPos] = "\f" # formfeed assumed not in text, and used as a placeholder
  }
  tex = paste(tex, collapse='')

  # LaTex and shinyTex commands that do not generate Shiny code  !!!! ????? DOES NOT HANDLE NESTING !!!! ????
  # handle \textbf, \textem, \shinyTip, \shinyLink   (with 1 to 2 "options" in braces)
  tex = gsub("(``|'')", '"', tex)
  tex = gsub("[\\](textbfem|textembf)[[:blank:]]*[{](.+?)[}]", "<strong><em>\\2</em></strong>", tex)
  tex = gsub("[\\]textbf[[:blank:]]*[{](.+?)[}]", "<strong>\\1</strong>", tex)
  tex = gsub("[\\]textem[[:blank:]]*[{](.+?)[}]", "<em>\\1</em>", tex)
  tex = gsub("[\\]textsc[[:blank:]]*[{](.+?)[}]", "<span style='font-variant:small-caps'>\\1</span>", tex)
  tex = gsub("[\\]tooltip([^[:alpha:]])", "\\\\shinyTip\\1", tex)
  tex = gsub("[\\]href([^[:alpha:]])", "\\\\shinyLink\\1", tex)
  tex = gsub("[\\]h([1-4])[[:blank:]]*[{](.*)[}]", "<h\\1>\\2</h\\1>", tex)
  tex = gsub("[\\]tooltipHref([^[:alpha:]])", "\\\\shinyLink\\1", tex)
  tex = gsub("[\\]shinyTip[[:blank:]]*[{](.+?)[}][[:blank:]]*[{](.+?)[}]",
             "<span title='\\2'><strong><em>\\1</em></strong></span>", tex)
  tex = gsub("[\\]shinyLink[[:blank:]]*[{](.+?)[}][[:blank:]]*[{]([^[:blank:]]+?)[}][[:blank:]]*[{](.+?)[}]",
             "<span title='\\2'><a href='\\3' target='_blank'>\\1</a></span>", tex)
  tex = gsub("[\\]shinyLink[[:blank:]]*[{](.+?)[}][[:blank:]]*[{]([^[:blank:]]+?)[}]",
             "<a href='\\2' target='_blank'>\\1</a>", tex)
  # inside tooltips, multiple spaces show as multiple spaces
  tex = gsub("[[:blank:]]{2,}", " ", tex)
  tex = gsub("[{][[:blank:]]*[\\](bfem|embf)[[:blank:]]+(.*?)[}]", "<strong><em>\\1</em></strong>", tex)
  tex = gsub("[{][[:blank:]]*[\\]bf[[:blank:]]+(.*?)[}]", "<strong>\\1</strong>", tex)
  tex = gsub("[{][[:blank:]]*[\\]em[[:blank:]]+(.*?)[}]", "<em>\\1</em>", tex)
  tex = gsub("[{][[:blank:]]*[\\]sc[[:blank:]]+(.*?)[}]", "<span style='font-variant:small-caps'>\\1</span>", tex)
  tex = gsub("[\\]item[[:blank:]]+", "<li> ", tex)

  # final check for Latex commands
  #if (length(grep("Scient",tex))>0) browser()
  finalCheck = gregexpr("[\\]([[:alpha:]]+)", tex)[[1]]
  #finalOK 
  if (finalCheck[1] != -1) {
    strings = substring(tex, finalCheck, finalCheck+attr(finalCheck,"match.length")-1)
    #worries = grep("[\\]shiny", strings, invert=TRUE)
    #if (any(worries)) {
      worries = 1:length(strings)  #????!!! remove ????!!!!!
      warning(texRange, ' un-handled LaTeX commands: ', strings[worries], 
              ifelse(length(worries==1), '', paste(' (and ', length(worries)-1, ' others')))
    #}
  } # end checking for un-handle LaTeX
  
  # restore mathJax
  if (hasMathJax) {
    formfeed = gregexpr("\f", tex)[[1]]
    assure(length(formfeed) == length(holdPos), 83645)
    for (i in 1:length(holdPos)) tex = sub("\f", holdTex[i], tex, fixed=TRUE)
  }
  
  
  return(tex)
} # end processText()

